/* For au format specs, see https://en.wikipedia.org/wiki/Au_file_format

   30 August 2023, by Ruud Harmsen.
   For the reason for writing this, see
   https://rudhar.com/musica/hwedwortehun/ia04.htm on my website.
   */
#include <stdio.h>
#include <math.h>

#define SampleRate 44100.0
#define Ampl       100

#define Freq1 (198 * 2.0 / 24 * 24.0)
#define Freq2 (198 * 2.0 / 24 * 27.0)
#define Freq3 (198 * 2.0 / 24 * 28.0)

#define Peri1 (SampleRate / Freq1)
#define Peri2 (SampleRate / Freq2)
#define Peri3 (SampleRate / Freq3)

#define Peri1i (int)Peri1
#define Peri2i (int)Peri2
#define Peri3i (int)Peri3

void makeBigEndian32 (int n, unsigned char *buf);

int main (void)
{
   char int32buf[4];

   fwrite(".snd", 4, 1, stdout);
   makeBigEndian32(28, int32buf);
   fwrite(int32buf, 4, 1, stdout);
   makeBigEndian32(0xffffffff, int32buf);
   fwrite(int32buf, 4, 1, stdout);
   /* 8-bit linear PCM */
   makeBigEndian32(2, int32buf);
   fwrite(int32buf, 4, 1, stdout);
   /* Sample rate */
   makeBigEndian32(SampleRate, int32buf);
   fwrite(int32buf, 4, 1, stdout);
   /* Just one mono channel */
   makeBigEndian32(1, int32buf);
   fwrite(int32buf, 4, 1, stdout);
   /* Empty annotation */
   makeBigEndian32(0, int32buf);
   fwrite(int32buf, 4, 1, stdout);

   /* Now write the data */
   int i, j;
   signed char s;

   for (j = 0; j < 4; j++)
   {
      /* Write values for two seconds */
      for (i = 0; i < SampleRate * 2; i++)
      {
         s = (int)(0.5 + ((i % Peri1i) / Peri1) * Ampl - (Ampl / 2))
           + (int)(0.5 + ((i % Peri2i) / Peri2) * Ampl - (Ampl / 2));
         fwrite(&s, 1, 1, stdout);
      }
      /* Same for the other pair of notes */
      for (i = 0; i < SampleRate * 2; i++)
      {
         s = (int)(0.5 + ((i % Peri1i) / Peri1) * Ampl - (Ampl / 2))
           + (int)(0.5 + ((i % Peri3i) / Peri3) * Ampl - (Ampl / 2));
         fwrite(&s, 1, 1, stdout);
      }
   }
   return 0;
}

void makeBigEndian32 (int n, unsigned char *buf)
{
   buf[3] = (n & 0x000000ff) >>  0;
   buf[2] = (n & 0x0000ff00) >>  8;
   buf[1] = (n & 0x00ff0000) >> 16;
   buf[0] = (n & 0xff000000) >> 24;
}
